//---------------------------------------------------------------------------
#ifndef vsTAPIH
#define vsTAPIH

//---------------------------------------------------------------------------
#include <Classes.hpp>
#include <Tapi.h>
#include <memory>

#include "vsTAPIThread.h"
#include "vsTAPISoundThread.h"

#define LINEUNAVAIL   "Line Unavailable"
#define LINEUNNAMED   "Line Unnamed"
#define LINENAMEEMPTY "Line Name is Empty"

#define HiVer 0x00020000 // Highest API version wanted (2.0)
#define LoVer 0x00010004 // Lowest API version accepted (1.4)


typedef void __fastcall (__closure *TTAPIError)(AnsiString asErrorDesc, int iErrorSection, int iFunctionID);
typedef void __fastcall (__closure *TCallerIDEvent)(AnsiString asName, AnsiString asNumber, int iCallType, HCALL hCall);
typedef void __fastcall (__closure *TCallRingEvent)(int iRingCount, const HCALL &hCall);
typedef void __fastcall (__closure *TCalledPartyIDEvent)(AnsiString asName, AnsiString asNumber, int iCallType);
typedef void __fastcall (__closure *TCallConnectedEvent)(AnsiString asAppName, DWORD dwBearerMode,
                                                         DWORD dwMediaMode, DWORD dwRate,
                                                         AnsiString asCalledPartyName,
                                                         AnsiString asComments, HCALL hCall);
typedef void __fastcall (__closure *TCallDisconnectedEvent)(HCALL hCall);

typedef void __fastcall (__closure *TCallInfoEvent)(LINEMESSAGE *cbData, LINECALLINFO *pLineCallInfo);
typedef void __fastcall (__closure *TCallLineReplyEvent)(int iRequestID, int iStatus);
typedef void __fastcall (__closure *TCallOfferingEvent)(HCALL hCall, int iOfferingMode, int iCallPrivaledge);

#pragma option push -b-
const int lcpNONE = LINECALLPRIVILEGE_NONE;
const int lcpMONITOR = LINECALLPRIVILEGE_MONITOR;
const int lcpOWNER = LINECALLPRIVILEGE_OWNER;
const int lcpMONITOR_AND_OWNER = LINECALLPRIVILEGE_MONITOR | LINECALLPRIVILEGE_OWNER;

const int lmmUNKNOWN = LINEMEDIAMODE_UNKNOWN;
const int lmmINTERACTIVEVOICE = LINEMEDIAMODE_INTERACTIVEVOICE;
const int lmmAUTOMATEDVOICE = LINEMEDIAMODE_AUTOMATEDVOICE;
const int lmmDATAMODEM = LINEMEDIAMODE_DATAMODEM;
const int lmmG3FAX = LINEMEDIAMODE_G3FAX;
const int lmmTDD = LINEMEDIAMODE_TDD;
const int lmmG4FAX = LINEMEDIAMODE_G4FAX;
const int lmmDIGITALDATA = LINEMEDIAMODE_DIGITALDATA;
const int lmmTELETEX = LINEMEDIAMODE_TELETEX;
const int lmmVIDEOTEX = LINEMEDIAMODE_VIDEOTEX;
const int lmmTELEX = LINEMEDIAMODE_TELEX;
const int lmmMIXED = LINEMEDIAMODE_MIXED;
const int lmmADSI = LINEMEDIAMODE_ADSI;
const int lmmVOICEVIEW = LINEMEDIAMODE_VOICEVIEW;

#pragma option pop

//---------------------------------------------------------------------------
class PACKAGE TvsTAPI : public TComponent
{
private:

  AnsiString asDebugLog_m;

  LPLINEDEVCAPS LnDvCpStruct_m;           // The structure
  LPLINEADDRESSCAPS LnAddrCpStruct_m;           // The structure

  HLINEAPP hLineApp_m;
  HLINEAPP hLineApp2_m;
  HLINE    hLine_m;
  HLINE    hLine2_m;
  unsigned long uiDeviceCount_m;
  unsigned long uiTapiVersion_m;

  AnsiString asAppTitle_m;

  DWORD dwTAPIDeviceID_m;
  AnsiString asTAPIVersionString_m;

  LINEEXTENSIONID tapiLEID_m;
  LINEEXTENSIONID tapiLEID2_m;

  TTAPIError     pOnTapiError_m;
  TCallerIDEvent pOnCallerIDEvent_m;
  TCalledPartyIDEvent pOnCalledPartyIDEvent_m;
  TCallConnectedEvent pOnCallConnectedEvent_m;
  TCallDisconnectedEvent pOnCallDisconnectedEvent_m;
  TCallInfoEvent pOnCallInfoEvent_m;
  TCallRingEvent pOnCallRingEvent_m;
  TCallLineReplyEvent pOnCallLineReplyEvent_m;
  TCallOfferingEvent pOnCallOfferingEvent_m;

  int setPrivileges_m;
  int setMediaModes_m;

  // This is a variably sized structure and gets handled in a special way
  // in terms of memory allocation etc..
  LINECALLINFO *pCallInfo_m;

  std::auto_ptr<vsTAPIThread> pTAPIThread_m;
  std::auto_ptr<vsTAPIThread> pTAPIThread2_m;

  HANDLE hKillThreadEventHandle_m;
  HANDLE hKillThreadEventHandle2_m;

  std::auto_ptr<vsTAPISoundThread> pRecordSound_m;

  void __fastcall DispatchCallEvent(LINEMESSAGE *cbData);
  DWORD __fastcall Get_The_LineDevCaps(HLINEAPP &hLineApp, DWORD &uiTapiVersion, int &iDeviceID);
  DWORD __fastcall Get_The_LineAddressCaps(HLINEAPP &hLineApp, DWORD &uiTapiVersion, int &iDeviceID);

  bool __fastcall CheckAllowedAutomatedVoice();

  DWORD __fastcall FillTAPILine(TStrings *sl, int NumDevs);
  DWORD __fastcall I_lineNegotiateLegacyAPIVersion(DWORD dwDeviceID);
  DWORD __fastcall I_lineNegotiateAPIVersion(DWORD dwDeviceID);
  void __fastcall TapiCheck(int ResultCode);

  void __fastcall LogDebug(AnsiString asData);

  bool __fastcall ProcessMessage(TMsg &Msg);
  void __fastcall ProcessMessages();

protected:

public:
  int __fastcall GetTAPIDevice(unsigned int uiDeviceCount);

  LPLINEDEVCAPS __fastcall GetDeviceCapabilities(DWORD &dwTAPIDeviceID,
                                                 bool bPromptEnumTAPIDevList = false);

  LPLINEADDRESSCAPS __fastcall GetAddressCapabilities(DWORD &dwTAPIDeviceID,
                                                      bool bPromptEnumTAPIDevList);

  bool __fastcall OpenDevice(DWORD &dwTAPIDeviceID, bool bPromptEnumTAPIDevList = false,
                             bool bIgnoreErrors=false);
  bool __fastcall CloseDevice();

  bool __fastcall MakeACall(AnsiString asName, AnsiString asPhoneNumber,
                            int iCountryCode, bool bBlockCallerID, int iCallType,
                            HCALL *hCall = NULL);

  bool __fastcall BlockCall(HCALL hCall);

  bool __fastcall PlayASound(AnsiString asSoundFile, AnsiString *asMsg=NULL);
  //bool __fastcall PlayASound(AnsiString asSoundFile, HCALL &hCall, AnsiString *asMsg=NULL);

  bool __fastcall RecordASound(AnsiString asSoundFile, HCALL &hCall,
                               int iMaxRecordSeconds=MAX_RECORD_CHUNKS*SOUND_BUFFER_SIZE,
                               AnsiString *asMsg=NULL,
                               int *iRecordedSecs=NULL);
  void __fastcall StopRecording();

  LONG __fastcall AnswerTheLine(HCALL &hCall, AnsiString *asErr=NULL);
  LONG __fastcall CloseCall(HCALL &hCall);

  AnsiString GetLineName(LPLINEDEVCAPS lpLineDevCaps_);

  LONG __fastcall vsGetCallStatus(HCALL &hCall, LINECALLSTATUS **pLinecallstatus, AnsiString *asTAPIErr=NULL);

  LONG __fastcall CheckForDialtone(HCALL &hCall);
  AnsiString __fastcall GetTapiError(LONG &lErr);
  LONG __fastcall GetCOMMPORTID(HANDLE &hComm, HCALL &hCall,LPVARSTRING lpVarString=NULL);

  __fastcall TvsTAPI(TComponent* Owner);
  __fastcall ~TvsTAPI();

__published:

  __property TTAPIError OnTapiError = { read = pOnTapiError_m, write = pOnTapiError_m };
  __property TCallerIDEvent OnCallerIDEvent = { read = pOnCallerIDEvent_m, write = pOnCallerIDEvent_m };
  __property TCalledPartyIDEvent OnCalledPartyIDEvent = { read = pOnCalledPartyIDEvent_m, write = pOnCalledPartyIDEvent_m };
  __property TCallConnectedEvent OnCallConnectedEvent = { read = pOnCallConnectedEvent_m, write = pOnCallConnectedEvent_m };
  __property TCallDisconnectedEvent OnCallDisconnectedEvent = { read = pOnCallDisconnectedEvent_m, write = pOnCallDisconnectedEvent_m };

  __property TCallRingEvent OnCallRingEvent = { read = pOnCallRingEvent_m, write = pOnCallRingEvent_m };
  __property TCallInfoEvent OnCallInfoEvent = { read = pOnCallInfoEvent_m, write = pOnCallInfoEvent_m };

  __property TCallLineReplyEvent OnCallLineReplyEvent = { read = pOnCallLineReplyEvent_m, write = pOnCallLineReplyEvent_m };
  __property TCallOfferingEvent OnCallOfferingEvent = { read = pOnCallOfferingEvent_m, write = pOnCallOfferingEvent_m };

  __property int Privileges = { read = setPrivileges_m, write = setPrivileges_m, default = lcpMONITOR_AND_OWNER };
  __property int MediaModes = { read = setMediaModes_m, write = setMediaModes_m, default = lmmINTERACTIVEVOICE };

  __property bool AutomatedVoiceAllowed = { read = CheckAllowedAutomatedVoice };
  __property AnsiString TAPIVersionString = { read = asTAPIVersionString_m };

  __property DWORD OpenDeviceID = { read = dwTAPIDeviceID_m };

  __property unsigned long TAPIDevicesDetected = { read = uiDeviceCount_m };

  __property AnsiString DebugLogFile = { read = asDebugLog_m, write = asDebugLog_m };

  __property AnsiString AppTitle = { read = asAppTitle_m, write = asAppTitle_m };

BEGIN_MESSAGE_MAP

  //MESSAGE_HANDLER(WM_CALLINFO_EVENT, TMessage, SignalCallEvent)

END_MESSAGE_MAP(TComponent)

};
//---------------------------------------------------------------------------

#endif
