//---------------------------------------------------------------------------
#ifndef vsTAPISoundThreadH
#define vsTAPISoundThreadH

//---------------------------------------------------------------------------
#include <Classes.hpp>

#include <mmsystem.h>
#include <Tapi.h>


#define SOUND_BUFFER_SIZE 1       // # seconds to buffer sound
#define MAX_RECORD_CHUNKS 60 * 10 // multipy with a 1 second buffer = 60 * 10 = 10 mins


typedef struct  // Used in playSound
{
     HMMIO         hndlMMio;         // handle to opened wave FILE
     PWAVEFORMATEX lpWaveFormat; // pointer to mem for fmt data
     LPSTR lpWaveData;         // the mem for ALL WAVE data
     PWAVEHDR lpWaveHdr;      // pointer to mem for WAVEHDR struc
     HWAVEOUT hWavePDevice;      // handle from waveOutOpen() output device

     DWORD dwWaveOutID;         // 0,1,2.. phonedeviceID as wavedevice
                                       // FLAGS
     bool bNeedErrWaveShutdown,             // for try finally
     bWaveFileIOOpen,                  // for hndlMMio
     bWaveFormatMemAloc,               // for lpWaveFormat
     bWaveDataMemAloc,                 // for lpWaveData
     bWaveHdrMemAloc,                  // for lpWaveHdr
     bWavePrepHeader,                  // waveOutPrepareHeader()
     bPlayWaveStarted,                 // has wave playing routine started
     bwaveOutOpen;                      // for hWaveODevice
                             // Keep on separate line
} MMPlayWvStruct;


#define PLAYSTATE_IDLE					1
#define PLAYSTATE_PASSWORD			2
#define PLAYSTATE_PLAYBACK			3
#define PLAYSTATE_RECORD				4
#define PLAYSTATE_RECORD_STOP		5
#define PLAYSTATE_SAVING        6

#define PLAYSUBSTATE_PASSWORD0			1
#define PLAYSUBSTATE_PASSWORD1			2
#define PLAYSUBSTATE_PASSWORD2			3
#define PLAYSUBSTATE_PASSWORD3			4

#define RINGCNT							3

#define IDM_AUTOANSWER 1

#define VS_PLAY                     0
#define VS_RECORD                   1


class vsTAPISoundThread : public TThread
{
private:

  int iAction_m;

  int iPlayState_m;
  int iPlaySubState_m;
  int nMsgCnt_m;
  int nPlayCnt_m;

  int iMaxRecordSeconds_m;
  int iRecordedBufferCount_m;
  int iLastRecordedBufferCount_m;

  HCALL  hCall_m;
  HANDLE h_waveout_m;
  HANDLE h_wavein_m;

  WAVEHDR *lpWaveHdr_m[MAX_RECORD_CHUNKS];
  HANDLE  hData_m[MAX_RECORD_CHUNKS];
  DWORD dwTotalBytesRecorded_m;

  DWORD dwWaveOutID_m;
  DWORD dwWaveInID_m;


  DWORD nRingCnt_m;

  bool bWaitForCall_m;
  bool bLineopen_m;

  static double ampavg;
  static const double threshold; // this probably should depend on the amount of bg noise detected for each call
  static const double falloffrate; // lower number shortens silence period - higher number lengthens it


  char szBuff_m[4096];
  AnsiString asTAPIErr_m;

  bool bSetupWasCalled_m;
  bool bIsRunning_m;
  bool bReturnSuccess_m;

  AnsiString asSoundFile_m;
  HLINE hLine_m;

  HWND hHiddenMsgWindow_m;
  HANDLE h_file_m;

  double dAmpavg_m;
  double dThreshold_m;   // this probably should depend on the amount of bg noise detected for each call
  double dFalloffrate_m; // lower number shortens silence period - higher number lengthens it

  void StopConnectedStuff(LPWAVEHDR lpWaveHdr,HWAVEOUT hWaveOut, LPSTR lpData, bool bResetCloseWavOut=true);
  LONG vslineGetWaveID(DWORD dwSelect, LPSTR waveDev);

  HWAVEIN recordMessage(DWORD dw_devid);
  int saveRecording(char *name, LPWAVEHDR hdr, bool bDone = true);
  void __fastcall PlayWaveFile();
  //HWAVEOUT playSound(DWORD dw_devid, char *szWaveFile);

  void __fastcall MessageHandler(TMessage &Msg);

  // Silence Detection Start

  // call this at beginning of telephone call
  //inline void Reset( void ) { dAmpavg_m = 100000.0; }

  // call this for each sample - can be inlined for faster performance
  /*
  inline bool IsQuiet( int sample)
  {
    dAmpavg_m = ( dAmpavg_m * dFalloffrate_m ) + (double) abs( sample );
    if ( dAmpavg_m < dThreshold_m )
      return true;

    return false;
  }
  */

  double __fastcall IsSilenceMonoSample(LPSTR lpWaveBuffer, LPWAVEFORMATEX lpWaveFormat, DWORD pos);

  //void __fastcall SetupSilenceDetection();

  // Silence Detection end

  int __fastcall GetRecordedSecs()
  {
    return(iLastRecordedBufferCount_m * SOUND_BUFFER_SIZE);
  }

  void __fastcall ResetSilence();
  bool __fastcall IsQuiet(int sample);

protected:
  void __fastcall Execute();

public:
  __fastcall vsTAPISoundThread(bool CreateSuspended);
  __fastcall ~vsTAPISoundThread();

  __property bool IsRunning = { read = bIsRunning_m };
  __property bool ReturnSuccess = { read = bReturnSuccess_m };
  __property AnsiString ErrorMsg = { read = asTAPIErr_m };

  void SetupSound(AnsiString SoundFile, HLINE hLine) { asSoundFile_m = SoundFile; hLine_m = hLine; bSetupWasCalled_m = true; iAction_m = VS_PLAY; };
  void SetupSound(AnsiString SoundFile, HLINE hLine, HCALL hCall, int iAction=VS_PLAY,int iMaxRecordSeconds=MAX_RECORD_CHUNKS*SOUND_BUFFER_SIZE) { asSoundFile_m = SoundFile; hLine_m = hLine; bSetupWasCalled_m = true; hCall_m = hCall; iAction_m = iAction; iMaxRecordSeconds_m = iMaxRecordSeconds; };

  void __fastcall StopRecording();

  __property int RecordedSeconds = { read = GetRecordedSecs };
};
//---------------------------------------------------------------------------
#endif
